import fire
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from radixhopper import BaseConverter, ConversionInput, ConversionError

console = Console()

def convert(num, base_from, base_to):
    try:
        input_data = ConversionInput(num=str(num), base_from=base_from, base_to=base_to)
        result = BaseConverter.base_convert(input_data)
        
        if '[' in result and ']' in result:
            parts = result.split('[')
            non_repeating = parts[0]
            repeating = parts[1].strip(']')
            formatted_result = Text()
            formatted_result.append(non_repeating)
            formatted_result.append(repeating, style="overline")
        else:
            formatted_result = result

        panel = Panel(formatted_result, title="Conversion Result", expand=False)
        console.print(panel)
    except ConversionError as e:
        console.print(f"[bold red]Error:[/bold red] {str(e)}")
    except Exception as e:
        console.print(f"[bold red]An unexpected error occurred. Please check your input and try again. Details:[/bold red]\n{str(e)}")

def main():
    fire.Fire(convert)

if __name__ == '__main__':
    main()


import os
import sys
import subprocess
import radixhopper

def find_st_path():
    # Get the directory of the radixhopper package
    radixhopper_dir = os.path.dirname(radixhopper.__file__)
    # Construct the path to st.py
    st_path = os.path.join(radixhopper_dir, 'st.py')
    return st_path

def run_streamlit_app():
    st_path = find_st_path()
    
    if not os.path.exists(st_path):
        print(f"Error: {st_path} not found.")
        return

    commands = [
        [sys.executable, '-m', 'streamlit', 'run', st_path],
        ['streamlit', 'run', st_path]
    ]

    for cmd in commands:
        try:
            subprocess.run(cmd, check=True)
            return
        except subprocess.CalledProcessError:
            print(f"Command failed: {' '.join(cmd)}")
        except FileNotFoundError:
            print(f"Command not found: {cmd[0]}")

    print("All attempts to run Streamlit app failed.")

if __name__ == "__main__":
    run_streamlit_app()


# SPDX-FileCopyrightText: 2024-present Aarmn the limitless <aarmn80@gmail.com>
#
# SPDX-License-Identifier: MIT

from .radixhopper import *
from .gui_runner import run_streamlit_app

__all__ = [ 
    "BaseConverter",
    "ConversionInput",
    "ConversionError",
    "InputError",
    "BaseRangeError",
    "DigitError",
    "run_streamlit_app"
]


from typing import Optional
from fractions import Fraction
from math import gcd
from pydantic import BaseModel, Field, validator
from enum import Enum

class ConversionError(Exception):
    pass

class InputError(ConversionError):
    pass

class BaseRangeError(ConversionError):
    pass

class DigitError(ConversionError):
    pass

class BaseRange(Enum):
    MIN = 2
    MAX = 36

class ConversionInput(BaseModel):
    num: str = Field(..., description="Number to convert")
    base_from: int = Field(..., ge=BaseRange.MIN.value, le=BaseRange.MAX.value, description="Base to convert from")
    base_to: int = Field(..., ge=BaseRange.MIN.value, le=BaseRange.MAX.value, description="Base to convert to")
    digits: str = Field(default="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", description="Digit set")

    @validator('num')
    def validate_num(cls, v, values):
        if 'base_from' in values and 'digits' in values:
            base_from = values['base_from']
            digits = values['digits']
            if not all(d in digits[:base_from] for d in v.upper() if d not in ".[]-"):
                raise DigitError(f"Invalid digit(s) for base {base_from}")
        return v.upper()

class BaseConverter:
    @staticmethod
    def _int_out(fraction: Fraction) -> tuple[int, Fraction]:
        i, frac = divmod(fraction.numerator, fraction.denominator)
        return i, Fraction(frac, fraction.denominator)

    @staticmethod
    def _any_to_frac(i: str, fp: str, fp_rep: str, from_base: int, digits: str) -> Fraction:
        fraction = Fraction()

        for index, digit in enumerate(i):
            fraction += digits.index(digit) * (from_base ** (len(i) - index - 1))
        fraction += (Fraction(BaseConverter._any_to_frac(fp, "", "", from_base, digits), from_base ** (len(fp)))) if fp else 0
        fraction += (Fraction(BaseConverter._any_to_frac(fp_rep, "", "", from_base, digits), ((from_base ** len(fp_rep)) - 1) * (from_base ** len(fp)))) if fp_rep else 0
        return fraction

    @staticmethod
    def _frac_to_any(x: Fraction, to_base: int, intpart: bool, digits: str) -> str:
        buffer_rep, out_x = "", ""
        while x > 0:
            if not intpart:
                if gcd(x.denominator, to_base) == 1:
                    if buffer_rep == "":
                        buffer_rep = x
                        out_x += "["
                    elif buffer_rep == x:
                        out_x += "]"
                        break
            x, digit = divmod(x, to_base) if intpart else BaseConverter._int_out(x * to_base)[::-1]
            out_x += digits[digit]
        return out_x[::-1] if intpart else out_x

    @staticmethod
    def base_convert(input_data: ConversionInput) -> str:
        int_str, fp_str, fp_rep_str = "", "", ""
        num = input_data.num
        if "." in num:
            i_str, fp_str = num.split(".")
            if ("[" in fp_str) or ("]" in fp_str):
                try:
                    fp_rep_str = fp_str[fp_str.index("[") + 1:fp_str.index("]")]
                    fp_str = fp_str[:fp_str.index("[")]
                except:
                    raise InputError("Invalid input format for repeating decimal")
        else:
            i_str = num

        i_int, f_frac = BaseConverter._int_out(BaseConverter._any_to_frac(i_str, fp_str, fp_rep_str, input_data.base_from, input_data.digits))
        return (BaseConverter._frac_to_any(i_int, input_data.base_to, True, input_data.digits) + 
                "." + BaseConverter._frac_to_any(f_frac, input_data.base_to, False, input_data.digits))
    

# handle zero


import streamlit as st
from radixhopper import BaseConverter, ConversionInput, ConversionError

def main():
    st.title("RadixHopper ✨")
    st.write("""Convert numbers between different bases (2-36). Fraction are supported and cyclic fractions are displayed with an overline.
    Use brackets in input to select the cyclic part, for example in base 10 to x, 0.0[3] would convert 1/30 to base x""")

    num = st.text_input("Enter number to convert:", value="0")
    base_from = st.number_input("Enter base to convert from:", min_value=2, max_value=36, value=10)
    base_to = st.number_input("Enter base to convert to:", min_value=2, max_value=36, value=10)

    if st.button("Convert"):
        try:
            input_data = ConversionInput(num=num, base_from=base_from, base_to=base_to)
            result = BaseConverter.base_convert(input_data)
            
            st.success("Conversion successful!")
            
            if '[' in result and ']' in result:
                parts = result.split('[')
                non_repeating = parts[0]
                repeating = parts[1].strip(']')
                formatted_result = f"Result: {non_repeating}<span style='text-decoration: overline;'>{repeating}</span>"
                st.markdown(formatted_result, unsafe_allow_html=True)
            else:
                st.markdown(f"Result: {result}")
        except ConversionError as e:
            st.error(f"Error: {str(e)}")
        except Exception as e:
            st.error(f"An unexpected error occurred: {str(e)}")

if __name__ == '__main__':
    main()